package cn.wisenergy.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.wisenergy.common.utils.DateUtil;
import cn.wisenergy.common.utils.ExcelUtils;
import cn.wisenergy.common.utils.exception.BASE_RESP_CODE_ENUM;
import cn.wisenergy.common.utils.exception.BaseCustomException;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.WorkHoliday;
import cn.wisenergy.model.app.WorkProject;
import cn.wisenergy.model.app.WorkUser;
import cn.wisenergy.model.dto.*;
import cn.wisenergy.model.enums.*;
import cn.wisenergy.service.StatisticsService;
import cn.wisenergy.service.WorkProjectService;
import cn.wisenergy.service.WorkUserService;
import com.alibaba.excel.util.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @author: nh
 * @create: 2021-01-19 17:19
 **/

@Service
@Slf4j
public class StatisticsServiceImpl implements StatisticsService {
    @Autowired
    private WorkUserService workUserService;

    @Autowired
    private WorkTimeOrderMapper workTimeOrderMapper;

    @Autowired
    private WorkUserDeptMapper workUserDeptMapper;

    @Autowired
    private WorkProjectService workProjectService;

    @Autowired
    private WorkHolidayMapper workHolidayMapper;

    @Autowired
    private WorkUserMapper workUserMapper;

    @Autowired
    private WorkProjectMapper workProjectMapper;

    @Autowired
    WorkProjectTimeCostMapper workProjectTimeCostMapper;

    private final static Integer EXCEL_WIDTH = 1000 * 6;

    private final static Integer DEFAULT_DEPT_ID = 1;

    @Override
    public PageInfo getDayWorkTimeOfMonth(GetMonthlyCollectParam param) {
        log.info("StatisticsServiceImpl[]getMonthlyCollect[]input.param" + param);
        if (param == null) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.INPUT_PARAM_IS_NULL);
        }
        //管理员用户
        WorkUser user = workUserService.getById(param.getUserId());
        if (!user.getLevel().equals(ManagerEnum.IS_SYSTEM_MANAGER.getCode())) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.THE_USER_NOT_MANAGER_PLASE_MANAGER_LOGIN);
        }
        //默认时间为当月
        Integer year = param.getYear();
        Integer month = param.getMonth();
        if (StringUtils.isEmpty(param.getYear()) || StringUtils.isEmpty(param.getMonth())) {
            Date now = new Date();
            year = DateUtil.getYear(now);
            month = DateUtil.getMonth(now) + 1;
        }
        //默认部门主键为1
        Integer deptId = param.getDeptId();
        if (param.getDeptId() == null) {
            deptId = DEFAULT_DEPT_ID;
        }
        //默认当前页码为1，每页数量为10
        if (param.getCurrentPage() == null) {
            param.setCurrentPage(1);
        }
        if (param.getSize() == null) {
            param.setSize(10);
        }

        // 分页获取部门下所有员工主键
        PageHelper.startPage(param.getCurrentPage(), param.getSize());
        List<Integer> userIds = workUserDeptMapper.listByDeptIdAndStatus(deptId);
        PageInfo pageInfo = new PageInfo(userIds);
        // 获取用户姓名、部门名称
        List<StatisticsTableDto> statisticsTableDtos = workUserMapper.getStatisticsTableDtos(userIds);
        // 获取用户当月每天工时信息
        List<DayWorkTimeAndType> dayWorkTimeAndTypes = workTimeOrderMapper.listByDateAndUserId(year, month, userIds);

        // 将工单集合与对应用户信息封装在一起
        Iterator<StatisticsTableDto> statisticsTableDtoIterator = statisticsTableDtos.iterator();
        while (statisticsTableDtoIterator.hasNext()) {
            StatisticsTableDto statisticsTableDto = statisticsTableDtoIterator.next();
            // 存放一位用户一个月中每天的工时信息
            ArrayList<DayWorkTimeAndType> dayWorkTimeAndTypes1 = new ArrayList<>();
            // 查询出的工时信息遍历
            Iterator<DayWorkTimeAndType> workTimeAndTypeIterator = dayWorkTimeAndTypes.iterator();
            while (workTimeAndTypeIterator.hasNext()) {
                DayWorkTimeAndType dayWorkTimeAndType = workTimeAndTypeIterator.next();
                if (dayWorkTimeAndType.getUserId().equals(statisticsTableDto.getUserId())) {
                    dayWorkTimeAndTypes1.add(dayWorkTimeAndType);
                }
            }
            statisticsTableDto.setDayWorkTimeAndTypeList(dayWorkTimeAndTypes1);
        }
        pageInfo.setList(statisticsTableDtos);
        return pageInfo;
    }

    @Override
    public List<Object> getMonthlyStatistics(Integer userId) {
        log.info("WorkTimeOrderServiceImpl[]getMonthlyStatistics[]input.param" + userId);
        WorkUser user = workUserService.getById(userId);
        //本月第一天
        Date firstDayOfMonth = DateUtil.getCurrentFirstDayOfMonth();
        //返回数据对象
        List<Object> objects = new ArrayList<>();
        //存放查询条件
        Map<String, Object> map = new HashMap<>();
        //用户是项目或部门级别，查询审批统计
        if (user.getLevel() != ManagerEnum.NOT_MANAGER.getCode()) {
            //以审批人、审核状态、开始时间查询本月审批数量
            map.put("reviewerId", userId);
            map.put("startDay", firstDayOfMonth);
            Integer completedCount = workTimeOrderMapper.getCountByCondition(map);

            //本月驳回未重报数量
            map.put("status", WorkOrderStatus.TURN_DOWN.getCode());
            Integer rejectAndNotResubmit = workTimeOrderMapper.getCountByCondition(map);

            // 项目级别，获取所管理项目,
            if (user.getLevel().equals(ManagerEnum.IS_PROJECT_DIRECTOR.getCode())) {
                List<WorkProject> projects = workProjectService.getUserManageProjects(userId);
                if (CollectionUtil.isNotEmpty(projects)) {
                    List<Integer> projectIds = projects.stream().map(WorkProject::getId).collect(Collectors.toList());
                    map.put("projectIds", projectIds);
                }
            }
            // 部门级别
            if (user.getLevel().equals(ManagerEnum.IS_DEPARTMENT_DIRECTOR.getCode())) {
                map.put("deptId", user.getDeptId());
                //填报项目类型编码
                int[] workOrderTypeArray = {WorkOrderType.EXTERNAL_WORK.getCode(), WorkOrderType.INTERNAL_WORK.getCode(), WorkOrderType.OTHER_NOT_PROJECT_BUSINESS.getCode()};
                map.put("typeArray", workOrderTypeArray);
            }
            //待审核包括：新提交和重新提交的
            List<Integer> statusArray = new ArrayList<>();
            statusArray.add(WorkOrderStatus.WAIT_AUDIT.getCode());
            statusArray.add(WorkOrderStatus.RE_SUBMIT.getCode());
            map.remove("reviewerId");
            map.remove("status");
            //本月未审批数量查询
            map.put("statusArray", statusArray);
            Integer notCompletedCount = workTimeOrderMapper.getCountByCondition(map);
            //所有查询结果封装进对象
            ApprovalStatistics approvalStatistics = new ApprovalStatistics();
            approvalStatistics.setApprovalCount(completedCount);
            approvalStatistics.setRejectNotSubmit(rejectAndNotResubmit);
            approvalStatistics.setNotApprovalCount(notCompletedCount);
            objects.add(approvalStatistics);
        }

        // 填报情况
        // 本月应填报日期
        List<Date> currentMonthWorkDays = this.currentMonthWorkDays(firstDayOfMonth);
        // 本月已填报日期
        List<Date> currentMonthFilledDays = currentMonthFilledDays(userId);
        // 本月未填报日期
        currentMonthWorkDays.removeAll(currentMonthFilledDays);
        int notFilledCount = currentMonthWorkDays.size();
        //本月被现存驳回数
        map.clear();
        map.put("status", 3);
        map.put("userId", userId);
        map.put("startDay", firstDayOfMonth);
        Integer countByCondition = workTimeOrderMapper.getCountByCondition(map);

        List<MonthlyWorkingHoursStatistics> monthlyWorkingHoursStatistics = workTimeOrderMapper.statisticsByProjectType(userId, firstDayOfMonth);
        // 如果没填写填报类型其中某项，造对象传前端
        while (monthlyWorkingHoursStatistics.size() < 5) {
            // 获取工时类型（项目或商机）
            List<String> projectTypes = monthlyWorkingHoursStatistics.stream().map(MonthlyWorkingHoursStatistics::getProjectType).collect(Collectors.toList());
            MonthlyWorkingHoursStatistics workingHoursStatistics = new MonthlyWorkingHoursStatistics();
            // 如果没填写项目
            if (!projectTypes.contains(ProjectTypeEnum.PROJECT.getTypeName())) {
                workingHoursStatistics.setProjectType(ProjectTypeEnum.PROJECT.getTypeName());
            }
            // 没填写商机
            if (!projectTypes.contains(ProjectTypeEnum.BUSINESS_OPPORTUNITY.getTypeName())) {
                workingHoursStatistics.setProjectType(ProjectTypeEnum.BUSINESS_OPPORTUNITY.getTypeName());
            }
            // 没填写外部商务
            if (!projectTypes.contains(ProjectTypeEnum.EXTERNAL_BUSINESS.getTypeName())) {
                workingHoursStatistics.setProjectType(ProjectTypeEnum.EXTERNAL_BUSINESS.getTypeName());
            }
            // 没填写内部培训、技术准备、管理
            if (!projectTypes.contains(ProjectTypeEnum.INTERNAL_TRAINING.getTypeName())) {
                workingHoursStatistics.setProjectType(ProjectTypeEnum.INTERNAL_TRAINING.getTypeName());
            }
            // 没填写其他非项目/商机工作
            if (!projectTypes.contains(ProjectTypeEnum.OTHER_NON_PROJECTS.getTypeName())) {
                workingHoursStatistics.setProjectType(ProjectTypeEnum.OTHER_NON_PROJECTS.getTypeName());
            }
            workingHoursStatistics.setCount(0);
            workingHoursStatistics.setWorkTime(0);
            monthlyWorkingHoursStatistics.add(workingHoursStatistics);
        }

        // 赋值未填报天数、被驳回未重报天
        for (MonthlyWorkingHoursStatistics monthlyWorkingHoursStatistic : monthlyWorkingHoursStatistics) {
            monthlyWorkingHoursStatistic.setNotFilledCount(notFilledCount);
            monthlyWorkingHoursStatistic.setRejectCount(countByCondition);
        }
        objects.add(monthlyWorkingHoursStatistics);
        return objects;
    }

    @Override
    public List<WorkTimeAndCostCollect> getCurrentMonthWorkTimeCollect(Integer userId, String startTime) {
        log.info("StatisticsServiceImpl[]getCurrentMonthWorkTimeCollect[]input.param" + userId + startTime);
        if (userId == null) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.INPUT_PARAM_IS_NULL);
        }
        // 时间不为空，赋值当月第一天
        if (StringUtil.isNotEmpty(startTime)) {
            Date currentFirstDayOfMonth = DateUtil.getCurrentFirstDayOfMonth();
            startTime = DateUtil.convertDateToYMDStr(currentFirstDayOfMonth);
        }
        WorkUser user = workUserService.getById(userId);
        // 普通用户
        if (user.getLevel().equals(ManagerEnum.NOT_MANAGER.getCode())) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.THE_USER_NOT_MANAGER_PLASE_MANAGER_LOGIN);
        }
        // 查询用户管理的项目
        List<WorkProject> userManageProjects = workProjectService.getUserManageProjects(userId);
        if (CollectionUtil.isEmpty(userManageProjects)) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.MANAGER_NOT_PROJECT);
        }
        // 获取项目集合中所有项目的项目主键
        List<Integer> projectIds = userManageProjects.stream().map(WorkProject::getId).collect(Collectors.toList());
        // 查询条件
        Map<String, Object> map = new HashMap<>();
        map.put("projectIds", projectIds);

        // 起始时间为空时不以时间作为查询条件
        if (StringUtil.isNotEmpty(startTime)) {
            map.put("firstDayOfMonth", startTime);
        }

        // 工时类型包含：项目，商机
        List<Integer> types = new ArrayList<>();
        types.add(ProjectTypeEnum.PROJECT.getType());
        types.add(ProjectTypeEnum.BUSINESS_OPPORTUNITY.getType());
        map.put("types", types);

        // 查询项目和商机的工时统计
        List<WorkTimeAndCostCollect> collect = workProjectTimeCostMapper.selectList(map);

        // 部门和中心级别
        if (!user.getLevel().equals(ManagerEnum.IS_PROJECT_DIRECTOR.getCode())) {
            // 请假、调休、外部商务、技术交流,内部培训、技术准备、管理,其他非项目/商机工作
            types.clear();
            map.clear();
            for (ProjectTypeEnum value : ProjectTypeEnum.values()) {
                // 查询条件排除项目和商机
                if (!value.getTypeName().equals(ProjectTypeEnum.PROJECT.getTypeName()) && !value.getTypeName().equals(ProjectTypeEnum.BUSINESS_OPPORTUNITY.getTypeName())) {
                    types.add(value.getType());
                }
            }

            if (user.getLevel().equals(ManagerEnum.IS_DEPARTMENT_DIRECTOR.getCode())) {
                map.put("deptId", user.getDeptId());
            }
            map.put("types", types);
            List<WorkTimeAndCostCollect> workTimeAndCostCollect = workTimeOrderMapper.getWorkTimeAndCostCollect(map);
            List<String> typeList = workTimeAndCostCollect.stream().map(WorkTimeAndCostCollect::getType).collect(Collectors.toList());
            // 如果查询结果不包含查询条件中的类型,并且不为项目或商机类型，造对象传前端
            for (ProjectTypeEnum value : ProjectTypeEnum.values()) {
                if (!typeList.contains(value.getTypeName()) && !value.getTypeName().equals(ProjectTypeEnum.PROJECT.getTypeName()) && !value.getTypeName().equals(ProjectTypeEnum.BUSINESS_OPPORTUNITY.getTypeName())) {
                    WorkTimeAndCostCollect workTimeAndCostCollect1 = new WorkTimeAndCostCollect();
                    workTimeAndCostCollect1.setType(value.getTypeName());
                    workTimeAndCostCollect.add(workTimeAndCostCollect1);
                }
            }
            collect.addAll(workTimeAndCostCollect);
        }
        for (WorkTimeAndCostCollect workTimeAndCostCollect : collect) {
            // 类型的总工时
            BigDecimal typeTotalTime = new BigDecimal("0.00");
            List<DeptProjectWorkTimeAndCost> deptProjectWorkTimeAndCosts = workTimeAndCostCollect.getDeptProjectWorkTimeAndCosts();
            if (CollectionUtil.isNotEmpty(deptProjectWorkTimeAndCosts)) {
                for (DeptProjectWorkTimeAndCost deptProjectWorkTimeAndCost : deptProjectWorkTimeAndCosts) {
                    List<ProjectWorkTimeAndCostStatistics> projectWorkTimeAndCostStatistics = deptProjectWorkTimeAndCost.getProjectWorkTimeAndCostStatistics();
                    for (ProjectWorkTimeAndCostStatistics projectWorkTimeAndCostStatistic : projectWorkTimeAndCostStatistics) {
                        typeTotalTime = typeTotalTime.add(projectWorkTimeAndCostStatistic.getTotalTime());
                    }
                }
            }
            workTimeAndCostCollect.setWorkTime(typeTotalTime);
        }
        return collect;
    }

    @Override
    public HSSFWorkbook exportWorkTimeExcel(GetMonthlyCollectParam param) {
        log.info("StatisticsServiceImpl[]exportWorkTimeExcel[]input.param{}" + param);
        PageInfo dayWorkTimeOfMonth = getDayWorkTimeOfMonth(param);
        Date date = DateUtil.convertStrToDate(param.getYear() + "-" + param.getMonth(), "yyyy-MM");
        // 传参的时间当月总天数
        int dayCount = DateUtil.getMonthDaysArray(date).length;
        // 需要导出的信息
        List<StatisticsTableDto> list = dayWorkTimeOfMonth.getList();
        // 创建工作簿
        HSSFWorkbook sheets = new HSSFWorkbook();
        // 创建表
        HSSFSheet sheet = sheets.createSheet("Sheet1");
        // 节假日样式，底色为红色,其余样式与普通项目
        HSSFCellStyle cellStyle1 = sheets.createCellStyle();
//        cellStyle1.setFillBackgroundColor(HSSFColor.HSSFColorPredefined.RED.getIndex());
        cellStyle1.setFillForegroundColor(HSSFColor.HSSFColorPredefined.RED.getIndex());
        // 设置全填充
        cellStyle1.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        // 普通样式对象
        HSSFCellStyle cellStyle = sheets.createCellStyle();
        // 垂直、水平中心对齐
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        cellStyle1.setVerticalAlignment(VerticalAlignment.CENTER);
        cellStyle1.setAlignment(HorizontalAlignment.CENTER);
        // 自动换行
        cellStyle.setWrapText(true);
        cellStyle1.setWrapText(true);
        // 第一列列宽
        sheet.setColumnWidth(1, EXCEL_WIDTH);
        // 获取本月应上班日期集合
        List<Date> dates = this.currentMonthWorkDays(date);
        List<Integer> days = new ArrayList<>();
        for (Date date1 : dates) {
            days.add(DateUtil.getDay(date1));
        }
        // 创建一行
        HSSFRow row1 = sheet.createRow(0);
        // 给第一行赋值
        row1.createCell(0).setCellValue("序号");
        row1.createCell(1).setCellValue("部门名称");
        row1.createCell(2).setCellValue("姓名");
        for (int i = 1; i <= dayCount; i++) {
            HSSFCell cell = row1.createCell(2 + i);
            cell.setCellValue(i + "日");
            // 如果为节假日
            if (!days.contains(i)) {
                cell.setCellStyle(cellStyle1);
            } else {
                cell.setCellStyle(cellStyle);
            }
        }
        Iterator<StatisticsTableDto> iterator = list.iterator();
        int i = 1;
        while (iterator.hasNext()) {
            StatisticsTableDto next = iterator.next();
            // 创建对应的行
            HSSFRow row_i = sheet.createRow(i);
            // 序号
            row_i.createCell(0).setCellValue(i);
            // 部门名称
            row_i.createCell(1).setCellValue(next.getDeptName());
            // 姓名
            row_i.createCell(2).setCellValue(next.getUserName());
            // 日期与工时集合
            List<DayWorkTimeAndType> timeAndTypes = next.getDayWorkTimeAndTypeList();
            // 在excel中对应日期单元格添加工时
            for (DayWorkTimeAndType timeAndType : timeAndTypes) {
                HSSFCell cell = row_i.getCell(timeAndType.getDay() + 2);
                String cellValue = ExcelUtils.getCellValue(cell, true);
                // 单元格为空则直接写入，否则追加
                if (!StringUtils.isEmpty(cellValue)) {
                    cell.setCellValue(timeAndType.getType() + timeAndType.getWorkTime().toString() + "\n" + cellValue);
                } else {
                    HSSFCell emptyCell = row_i.createCell(timeAndType.getDay() + 2);
                    emptyCell.setCellValue(timeAndType.getType() + timeAndType.getWorkTime().toString());
                }
            }
            // 给该行每个单元格设置单元格样式
            for (Cell cell : row_i) {
                cell.setCellStyle(cellStyle);
            }
            // 换行
            i++;
        }
        return sheets;
    }

    @Override
    public List<UserWorkTimeStatisticsByProject> getUserWorkTimeStatisticsReport(Integer userId, String startDate, String endDate, Integer status) {
        log.info("StatisticsServiceImpl[]getUserWorkTimeStatisticsReport[]");
        // 默认统计截至时间为前一天
        if (StringUtil.isEmpty(endDate)) {
            endDate = DateUtil.convertDateToYMDStr(new Date());
        }
        // 获取用户负责项目主键集合
        List<WorkProject> userManageProjects = workProjectService.getUserManageProjects(userId);
        if (CollectionUtil.isEmpty(userManageProjects)) {
//            throw new BaseCustomException(BASE_RESP_CODE_ENUM.MANAGER_NOT_PROJECT);
        }
        List<Integer> projectIds = userManageProjects.stream().map(WorkProject::getId).collect(Collectors.toList());

        List<UserWorkTimeStatisticsByProject> userWorkTimeStatisticsByProjects = new ArrayList<>();
        // 返回结果集合第一个元素是用户管理项目的集合
        UserWorkTimeStatisticsByProject userWorkTimeStatisticsByProject = new UserWorkTimeStatisticsByProject();
        List<ProjectWorkTimeAndType> projectWorkTimeAndTypes = new ArrayList<>();
        for (WorkProject userManageProject : userManageProjects) {
            ProjectWorkTimeAndType projectWorkTimeAndType = new ProjectWorkTimeAndType();
            projectWorkTimeAndType.setProjectId(userManageProject.getId());
            projectWorkTimeAndType.setProjectName(userManageProject.getProjectName());
            projectWorkTimeAndTypes.add(projectWorkTimeAndType);
        }
        userWorkTimeStatisticsByProject.setProjectWorkTimeAndType(projectWorkTimeAndTypes);
        userWorkTimeStatisticsByProjects.add(userWorkTimeStatisticsByProject);

        // 查询人员项目工时统计
        List<UserWorkTimeStatisticsByProject> userProjectWorkTimeStatistics = workTimeOrderMapper.getUserProjectWorkTimeStatistics(startDate, endDate, status, projectIds);
        // 取出单人的项目工时统计
        for (UserWorkTimeStatisticsByProject userProjectWorkTimeStatistic : userProjectWorkTimeStatistics) {
            int totalTime = 0;
            for (ProjectWorkTimeAndType projectWorkTimeAndType : userProjectWorkTimeStatistic.getProjectWorkTimeAndType()) {
                totalTime += projectWorkTimeAndType.getWorkTime();
            }
            userProjectWorkTimeStatistic.setTotalTime(totalTime);
        }
        // 将项目名称集合与人员工时统计封装在一起
        userWorkTimeStatisticsByProjects.addAll(userProjectWorkTimeStatistics);
        return userWorkTimeStatisticsByProjects;
    }

    @Override
    public List<ProjectStatisticsByMonth> getProjectStatistics(Integer userId, Integer deptId, Integer year, Integer projectId) {
        log.info("StatisticsServiceImpl[]getProjectStatistics[]input.param" + userId + deptId + year + projectId);
        // 权限判断

        // 默认查询时间为当前
        if (year == null) {
            year = DateUtil.getYear(new Date());
        }
        // 部门主键和项目主键不能同时为空
        if (deptId == null && projectId == null) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.INPUT_PARAM_IS_NULL);
        }

        // 部门主键参数不为空，查询部门下所有项目统计信息。为空时，查询项目编号参数为主键的项目统计信息
        List<ProjectStatisticsByMonth> projectStatisticsByMonths = workProjectTimeCostMapper.selectListByMonth(year, deptId, projectId);
        // 计算工时和成本合计
        if (CollectionUtil.isNotEmpty(projectStatisticsByMonths)) {
            // 存放所有项目所有月份合计
            ProjectStatisticsByMonth projectStatisticsByMonthTotal = new ProjectStatisticsByMonth();
            List<ProjectStatisticsByMonth.StatisticsDateAndWorkTime> statisticsDateAndWorkTimeList = new ArrayList<>(13);
            // 设置合计时间1到12月和项目合计
            for (int i = 0; i < 12; i++) {
                ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime = new ProjectStatisticsByMonth.StatisticsDateAndWorkTime();
                statisticsDateAndWorkTime.setDate(i + 1 + "月");
                statisticsDateAndWorkTime.setTotalTime(new BigDecimal("0.00"));
                statisticsDateAndWorkTime.setCost(new BigDecimal("0.00"));
                statisticsDateAndWorkTimeList.add(statisticsDateAndWorkTime);
            }
            ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime1 = new ProjectStatisticsByMonth.StatisticsDateAndWorkTime();
            statisticsDateAndWorkTime1.setDate("项目合计");
            statisticsDateAndWorkTimeList.add(statisticsDateAndWorkTime1);

            // 遍历所有项目的统计
            for (ProjectStatisticsByMonth projectStatisticsByMonth : projectStatisticsByMonths) {
                // 一个项目统计中的所有月份工时和成本信息
                BigDecimal totalTime = new BigDecimal("0.00");
                BigDecimal cost = new BigDecimal("0.00");
                for (ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTimes : projectStatisticsByMonth.getStatisticsDateAndWorkTimes()) {
                    for (ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime : statisticsDateAndWorkTimeList) {
                        // 某个项目的全周期生命统计不需要所有项目合计
                        if (statisticsDateAndWorkTime.getDate().equals(statisticsDateAndWorkTimes.getDate()) && deptId != null) {
                            statisticsDateAndWorkTime.setCost(statisticsDateAndWorkTime.getCost().add(statisticsDateAndWorkTimes.getCost()));
                            statisticsDateAndWorkTime.setTotalTime(statisticsDateAndWorkTime.getTotalTime().add(statisticsDateAndWorkTimes.getTotalTime()));
                        }
                    }
                    cost = cost.add(statisticsDateAndWorkTimes.getCost());
                    totalTime = totalTime.add(statisticsDateAndWorkTimes.getTotalTime());
                }
                ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime = new ProjectStatisticsByMonth.StatisticsDateAndWorkTime();
                statisticsDateAndWorkTime.setDate("项目合计");
                statisticsDateAndWorkTime.setCost(cost);
                statisticsDateAndWorkTime.setTotalTime(totalTime);
                // 将统计的结果放在一个项目统计集合的末尾
                projectStatisticsByMonth.getStatisticsDateAndWorkTimes().add(statisticsDateAndWorkTime);
            }
            if (deptId != null) {
                projectStatisticsByMonthTotal.setStatisticsDateAndWorkTimes(statisticsDateAndWorkTimeList);
                projectStatisticsByMonthTotal.setProjectName("合计");
                projectStatisticsByMonths.add(projectStatisticsByMonthTotal);
            }
        }

        return projectStatisticsByMonths;
    }

    //获取一个月应上班日期，如果是当月，时间截至到当天
    public List<Date> currentMonthWorkDays(Date date) {
        log.info("StatisticsServiceImpl[]currentMonthWorkDays[]input.method");
        // 时间为本月,起始时间为参数时间当月第一天，截至时间为当天。否则，截至时间为当月最后一天
        Date startDate = DateUtil.getFirstDayOfMonth(date);
        Date endDate = null;
        // 当前时间
        Date now = new Date();
        if (DateUtil.getYear(date) == DateUtil.getYear(now) && DateUtil.getMonth(date) == DateUtil.getMonth(now)) {
            endDate = now;
        } else {
            endDate = DateUtil.getLastDayOfMonth(date);
        }
        // 当月月第一天到截止时间的日期集合
        List<Date> workDays = DateUtil.getDatesBetweenTwoDate(startDate, endDate);

        //除去周末
        Iterator<Date> iterator = workDays.iterator();
        while (iterator.hasNext()) {
            Date next = iterator.next();
            int week = DateUtil.getWeek(next);
            // week为7是星期六，1是星期天
            if (week == 7 || week == 1) {
                iterator.remove();
            }
        }
        //获取当月的节日和周末上班日期
        List<WorkHoliday> specialDates = workHolidayMapper.getByDate(startDate, endDate);
        if (CollectionUtil.isEmpty(specialDates)) {
            return workDays;
        }
        for (WorkHoliday specialDate : specialDates) {
            List<Date> isWorkDays = DateUtil.getDatesBetweenTwoDate(specialDate.getDayStart(), specialDate.getDayEnd());
            //获取的是工作日
            if (specialDate.getDayType().equals(HolidayTypeEnum.WORKING_DAY.getCode())) {
                //在本月应上班集合中加上这天，一般都是放假前后的周末调整上班
                workDays.addAll(isWorkDays);
            } else {
                //否则就除去假日
                workDays.removeAll(isWorkDays);
            }
        }
        return workDays;
    }

    //获取用户本月已填报的日期
    public List<Date> currentMonthFilledDays(Integer userId) {
        log.info("StatisticsServiceImpl[]currentMonthFilledDays[]input.method");
        Date firstDayOfMonth = DateUtil.getCurrentFirstDayOfMonth();
        List<Date> currentMonthFilledDays = workTimeOrderMapper.getDaysByDateAndStatus(firstDayOfMonth, userId);
        return currentMonthFilledDays;
    }

}
